****************************************************************************
****************************************************************************
**                                                                        **
**   Phoenix for Fargo II : Basic Enemy Handling Routines                 **
**                                                                        **
**   Copyright 1997-2000 by Patrick Davidson.  All Rights Reserved.       **
**                                                                        **
**   This file was last updated April 19, 2000                            **
**                                                                        **
**   For complete copyright information and contact addresses, refer to   **
**   the main source file "PHOENIX.ASM"                                   **
**                                                                        **
****************************************************************************
****************************************************************************

******************************************** MOVE THE ENEMIES
*
* This enemy routine moves every enemy.  It uses the enemy's type in each
* enemy structure to select a routine from the table of actual enemy
* routines.  The table itself, as well as the routines, are in the file
* ENEMIES2.ASM.
*
* Each enemy routine will be called with A4 pointing to that enemy's enemy
* structure, and A3 pointing to the position in the coordinate table for
* that enemy (the coordinate table has 4 bytes per enemy, and is mainly used
* to specify locations for enemies that are in formation).  The D7 register
* is used as the loop counter; the order in which the enemies are processed
* is reversed at each frame, so the low bit will be different at each frame
* for a specific enemy.
*
* The enemy handling routines should preserver the A3, A4, and D7 registers.
* Their purpose is to move the enemy by updating its coordinates, and make
* other changes to the data as needed (e.g. changing the image if the enemy
* is animated).  It should NOT draw the enemy to the screen.
*
********

Move_Enemies:
         btst     #0,cycle_counter+3(a5)  
         beq.s    enemies_opposite

         move.l   xy_pointer(a5),a3          ;A3 -> standard coordinates
         lea      enemies_data(a5),a4        ;A4 -> enemy structure
         moveq    #num_en-1,d7
loop_move_enemies:
         move.w   (a4),d0              
         beq.s    me_no_enemy_here
         move.w   Enemy_Code_Table(pc,d0.w),d0
         jsr      Enemy_Code_Table(pc,d0.w)
me_no_enemy_here:
         lea      e_size(a4),a4
         addq.l   #4,a3
         dbra     d7,loop_move_enemies
         rts

enemies_opposite:
         move.l   xy_pointer(a5),a3
         add.l    #4*num_en,a3
         lea      enemies_data_end(a5),a4  
         moveq    #num_en-1,d7
_loop_move_enemies:
         lea      -e_size(a4),a4
         subq.l   #4,a3
         move.w   (a4),d0              
         beq.s    b_me_no_enemy_here
         move.w   Enemy_Code_Table(pc,d0.w),d0
         jsr      Enemy_Code_Table(pc,d0.w)
b_me_no_enemy_here:
         dbra     d7,_loop_move_enemies
         rts

******************************************** ENEMY TABLE & CONTROL ROUTINES

         INCLUDE  enemies2.asm

******************************************** DRAW/ERASE THE ENEMIES
*
* Quite simply, this runs through the enemy array, and draws each enemy on
* the screen using the standard sprite display code.
*
********

Draw_Enemies:
         lea      enemies_data(a5),a4
         moveq    #num_en-1,d7
loop_draw_enemies:
         tst.w    (a4)
         beq.s    ed_no_enemy_here
         move.w   e_y(a4),d1
         move.w   e_x(a4),d0
         move.w   e_image(a4),d2
         bsr      Xor_Sprite_D2
ed_no_enemy_here:
         lea      e_size(a4),a4
         dbra     d7,loop_draw_enemies
         rts

******************************************** HANDLE COLLISIONS WITH ENEMIES
*
* This routine tests for collisions between enemies and bullets fired by the
* player.  It tests every possible pair of such objects.  It assumes that
* all objects are rectangular, thus a collision is detected whenever all
* four of these conditions are met:
*
* (Y1 + H1 - Y2) >= 0
* (Y1 - Y2 - H2) <= 0
* (X1 + W1 - X2) >= 0
* (X1 - X2 - W2) <= 0
*
* Whenever a collision is detected, the enemy is normally damaged by the
* amount specified in the bullet, and the bullet is destroyed.  The only
* exception is the special bullet (with a negative type) which is not
* destroyed.
*
* When the enemy is destroyed, it is normally simply changed to an enemy of
* type -2 (an exploding enemy).  However, there must be special handling for
* the bosses (which have larger explosions) and especially for the super-
* bosses, which are actually composed of multiple enemies where the
* destruction of one affects the other.  I PLAN TO CHANGE THIS DESIGN IN THE
* NEXT VERSION OF THE GAME.
*
********

Enemy_Collisions:
         lea      bullets_data(a5),a3        ;A3 = Bullet pointer
         move.w   num_b(a5),d3               ;D3 = Bullet counter
EC_Process_Bullets:
         tst.w    (a3)
         beq.s    EC_PB_no_bullet_here
         lea      enemies_data(a5),a4        ;A4 = Enemy pointer
         moveq    #num_en-1,d4               ;D4 = Enemy counter
EC_Process_Enemies:
         tst.w    (a4)
         ble.s    EC_PE_not_here

         move.w   e_y(a4),d0
         add.w    e_h(a4),d0
         sub.w    b_y(a3),d0
         blt.s    EC_PE_not_here
         sub.w    e_h(a4),d0
         sub.w    b_h(a3),d0    
         bgt.s    EC_PE_not_here
         
         move.w   e_x(a4),d0
         add.w    e_w(a4),d0
         sub.w    b_x(a3),d0
         blt.s    EC_PE_not_here
         sub.w    e_w(a4),d0
         sub.w    b_w(a3),d0
         bgt.s    EC_PE_not_here

         move.w   b_dmg(a3),d0
         sub.w    d0,e_dmg(a4)
         bgt.s    EC_not_destroyed

         move.w   #-2,(a4)
         sf       e_data(a4)
         move.w   #in_Explosion1,e_image(a4)
         move.w   #32767,e_dmg(a4)
         cmp.w    #17,e_w(a4)
         bne.s    EC_not_destroyed
         bsr.s    boss_exploded

EC_not_destroyed:
         tst.w    (a3)
         ble.s    EC_PE_not_here
         clr.w    (a3)

         lea      b_size(a3),a3
         dbra     d3,EC_Process_Enemies
         rts

EC_PE_not_here:
         lea      e_size(a4),a4
         dbra     d4,EC_Process_Enemies
EC_PB_no_bullet_here:
         lea      b_size(a3),a3
         dbra     d3,EC_Process_Bullets
         rts

boss_exploded:
         move.w   #-4,(a4)
         move.w   #in_LExplosion1,e_image(a4)

         cmp.w    #18,-e_size(a4)            ; Previous/next superboss part
         bne.s    not_type_18_prev
         addq.w   #8,-e_size+e_image(a4)
         bra.s    boss_explode_done
not_type_18_prev:
         cmp.w    #20,e_size(a4)
         bne.s    boss_explode_done
         addq.w   #8,e_size+e_image(a4)

boss_explode_done:
         rts
